/***************************************************************************
                          apollonqueryview.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	:

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
using namespace std;

#include <qstring.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qtabwidget.h>
#include <qregexp.h>  
#include <qtooltip.h>

#include <kconfig.h>
#include <klistview.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <klocale.h>





#include "apollonqueryview.h"
#include "apollonsearchviewitem.h"
#include "apollonsearchviewitemchild.h"


ApollonQueryView::ApollonQueryView( QTabWidget* parent, const int ID, const int& index, const QString& text , giFTConnection* gcn):
	QVBox( (QWidget *) parent ),
	parent( parent )
{
	_ID = ID;
	_index = index;
	query = text;
	_gcn=gcn;

	KIconLoader *loader = KGlobal::iconLoader();

	QHBox* box = new QHBox(this);

	iconlabel = new QLabel(box);
	label = new QLabel(box);

	QString realm;

	switch (index)
	{
		case 0:
			realm = "everything";
			iconlabel->setPixmap( loader->loadIcon("find",KIcon::Small) );
			break;
		case 1:
			realm = "audio";
			iconlabel->setPixmap( loader->loadIcon("sound",KIcon::Small) );
			break;
		case 2:
			realm = "video";
			iconlabel->setPixmap( loader->loadIcon("video",KIcon::Small) );
			break;
		case 3:
			realm = "images";
			iconlabel->setPixmap( loader->loadIcon("image",KIcon::Small) );
			break;
		case 4:
			realm = "text documents";
			iconlabel->setPixmap( loader->loadIcon("txt",KIcon::Small) );	
			break;
		case 5:
			realm = "software";
			iconlabel->setPixmap( loader->loadIcon("binary",KIcon::Small) );
			break;
		
		case 6:
			// These are not allowed as realm	
			//realm = "user";
			iconlabel->setPixmap( loader->loadIcon("connect_established",KIcon::Small) );
			break;
		case 7:
			// These are not allowed as realm	
			//realm = "hash";
			iconlabel->setPixmap( loader->loadIcon("help_index",KIcon::Small) );
			break;
	}
	
	labeltext_search = "Results for <i>" + realm + "</i> with keywords <i>" + text + "</i> - searching...";
	labeltext_complete = "Results for <i>" + realm + "</i> with keywords <i>" + text + "</i> - Search completed!";
	
	label->setText( labeltext_search );
	

	more = new QPushButton(box);
	//stop->setPixmap(loader->loadIcon("player_pause", KIcon::Small));
	more->setText(i18n("Search More"));
	more->setFixedHeight( more->sizeHint().height() );
	more->setFixedWidth( more->sizeHint().width() );
	connect(more, SIGNAL(clicked()), this, SLOT(moreSearch()));
	QToolTip::add(more, "Try to find more sources for this query");
	more->setEnabled(0);

	stop = new QPushButton(box);
	//stop->setPixmap(loader->loadIcon("player_pause", KIcon::Small));
	stop->setText(i18n("Stop Search"));
	stop->setFixedHeight( stop->sizeHint().height() );
	stop->setFixedWidth( stop->sizeHint().width() );
	connect(stop, SIGNAL(clicked()), this, SLOT(stopSearch()));
	QToolTip::add(stop, "Stop search");

	QPushButton* close = new QPushButton(box);
	close->setPixmap(loader->loadIcon("fileclose", KIcon::Small));
	close->setFixedHeight( close->sizeHint().height() );
	close->setFixedWidth( close->sizeHint().width() );
	QToolTip::add(close, "Close tab");

	box->setStretchFactor(label, 4);
	box->setStretchFactor(close, 0);
	
	connect(close, SIGNAL(clicked()), this, SLOT(deleteThis()) );
	
	searchresults = new KListView(this);
	searchresults->setSelectionMode(QListView::Extended);
	searchresults->setShowSortIndicator(TRUE);
	searchresults->setAllColumnsShowFocus(TRUE);
	searchresults->setRootIsDecorated(TRUE);
	searchresults->setMinimumSize(300, 150);

	searchresults->addColumn("");
	searchresults->setColumnWidthMode(0, QListView::Manual);
	searchresults->setColumnWidth(0, 55);

	searchresults->addColumn("Name");
	searchresults->setColumnWidthMode(1, QListView::Manual);
	searchresults->setColumnWidth(1, 350);

	searchresults->addColumn("Size");
	searchresults->setColumnWidth(2,80);
	searchresults->setColumnAlignment(2, Qt::AlignRight);
	
	searchresults->addColumn("Slots");
	searchresults->setColumnWidth(3,80);
	searchresults->setColumnAlignment(3, Qt::AlignRight);
	
	searchresults->addColumn("User");
	searchresults->setColumnWidth(4,50);
	searchresults->setColumnAlignment(4, Qt::AlignRight);
	searchresults->setColumnWidthMode(4, QListView::Manual);

	searchresults->addColumn("Album");
	searchresults->setColumnWidth(5,80);
	searchresults->setColumnWidthMode(5, QListView::Manual);
	searchresults->setColumnAlignment(5, Qt::AlignLeft);

	searchresults->addColumn("Duration");
	searchresults->setColumnWidth(6,80);
	searchresults->setColumnAlignment(6, Qt::AlignRight);

	searchresults->addColumn("Bitrate");
	searchresults->setColumnWidth(7,80);
	searchresults->setColumnAlignment(7, Qt::AlignRight);


	searchresults->setSorting(2,FALSE);

	connect( searchresults, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(itemDoubleClicked(QListViewItem *) ) );

	connect( searchresults, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)), this, SLOT(queryPopup(QListViewItem*, const QPoint&, int)));

	// Menu if there is right=clicked on a parent
	menu = new  QPopupMenu(searchresults);
	
	// submenu of menu
	browseusermenu = new QPopupMenu(searchresults);
	menu->insertItem(loader->loadIcon("2downarrow", KIcon::Small), "Download", this, SLOT(sltDownload()));
	menu->insertItem(loader->loadIcon("folder", KIcon::Small), "Browse user:", browseusermenu); 	
	
	// Menu if there is rightclicked on a child
	sourcemenu = new QPopupMenu(searchresults);
	//sourcemenu->insertItem(loader->loadIcon("folder", KIcon::Small), "Browse this user", this, SLOT(browse()) );


	KConfig* config = KGlobal::config();
	searchresults->restoreLayout (config, "QueryView");
}

ApollonQueryView::~ApollonQueryView()
{
  KConfig* config = KGlobal::config();
  searchresults->setColumnWidth(1,350);
  // searchresults->setColumnWidth(0,30);
  searchresults->setColumnWidth(2,80);
  searchresults->setColumnWidth(3,80);
  searchresults->setColumnWidth(4,50);
  searchresults->setColumnWidth(5,80);
  searchresults->setColumnWidth(6,80);
  searchresults->setColumnWidth(7,80);
  searchresults->saveLayout (config, "QueryView");
  config->sync();
}


void ApollonQueryView::stopSearch() {
  emit giftCancelSearchRequest(_ID);
}
void ApollonQueryView::moreSearch() {
_ID = _gcn->getNewID();
_gcn->makeSearchRequest( _ID, _index, query );
label->setText( "Searching More...");
parent->changeTab(this, query + i18n(" (searching more)"));
more->setEnabled(0);
stop->setEnabled(1);

}

void ApollonQueryView::deleteThis()
{
	// Make a call to the deamon not to search for thisone anymore
	// FIXME: make the call
	
	emit giftCancelSearchRequest(_ID);
	
	delete this;
}

void ApollonQueryView::itemDoubleClicked(QListViewItem* item)
{
        

	ApollonSearchViewItem* svi  = static_cast<ApollonSearchViewItem*>(item);
	if( svi->childCount() == 0 ){
	item = item->parent();
	itemDoubleClicked(item);
	return;
	}
	ApollonSearchViewItemChild* svic = dynamic_cast<ApollonSearchViewItemChild*> (item) ;
	
	QListViewItem * child = svi->firstChild();
	ApollonSearchViewItemChild* castchild1 = static_cast<ApollonSearchViewItemChild*>(child);

	if ( svi )
	  {
	   
	    KIconLoader *loader = KGlobal::iconLoader();
	    QString  protocol_name = castchild1->getUrl();
	    protocol_name = protocol_name.left(protocol_name.find(":"));
	    if(protocol_name=="FastTrack")
	      {
		svi->setPixmap(0, loader->loadIcon("kazaadown", KIcon::Small));
	      }
	    else if(protocol_name=="OpenNap")
	      {
		svi->setPixmap(0, loader->loadIcon("napsterdown", KIcon::Small));
	      }
	    else if(protocol_name=="Gnutella")
	      {
		svi->setPixmap(0, loader->loadIcon("gnutelladown", KIcon::Small));
	      }
	    else if(protocol_name=="OpenFT")
	      {
		svi->setPixmap(0, loader->loadIcon("openftdown", KIcon::Small));
		//	QToolTip::add(svi->pixmap(0), "OpenFT-Network");
	      }
	    
	    QListViewItem * child = svi->firstChild();
	    
	    while( child ) 
	      {
		ApollonSearchViewItemChild* castchild = static_cast<ApollonSearchViewItemChild*>(child);
		castchild->setPixmap(0, loader->loadIcon("2downarrow", KIcon::Small));
		emit giftAddRequest( castchild->getUser(), castchild->getHash(), castchild->getSize(), castchild->getUrl(), castchild->getFileName() );	
		child = child->nextSibling();
		    }
	  }
	if ( svic )
	  {
	    KIconLoader *loader = KGlobal::iconLoader();
	    svic->setPixmap(0, loader->loadIcon("2downarrow", KIcon::Small));

	    emit giftAddRequest( svic->getUser(), svic->getHash(), svic->getSize(), svic->getUrl(), svic->getFileName() );
	  }
	
}


void ApollonQueryView::sltDownload()
{
       QListViewItem* item = searchresults->firstChild();
       while (item) 
	 {
	   if (item->isSelected()) 
	     itemDoubleClicked(item);
	   item = item->nextSibling();
	 }
}




void ApollonQueryView::queryPopup(QListViewItem* item, const QPoint & point, int col)
{
	current = "";
	
	if (!item)
		return;
	
	ApollonSearchViewItemChild* svic = dynamic_cast<ApollonSearchViewItemChild*> (item) ;
	ApollonSearchViewItem* svi = dynamic_cast<ApollonSearchViewItem*> (item) ;
	KIconLoader *loader = KGlobal::iconLoader();
	

	if ( svic )
	  {
	    sourcemenu->clear();
	    current = svic->getUser();
	    sourcemenu->insertItem(loader->loadIcon("folder", KIcon::Small), "Browse user : " + current, this, SLOT(browse()) );
	    sourcemenu->popup(point);
	  }

	else if ( svi )
	{
		// Loop through the children to get the user names out and put them in the menu
		
		browseusermenu->clear();
		
		QListViewItem * child = svi->firstChild();
	
		  while( child )
		    {
		      ApollonSearchViewItemChild* castchild = static_cast<ApollonSearchViewItemChild*>(child);
		      QString user = castchild->getUser();
		      browseusermenu->insertItem(loader->loadIcon("connect_established", KIcon::Small), user, this, SLOT(browse(int)) );
		      child = child->nextSibling();
		    }

		  menu->popup(point);
	}

}

void ApollonQueryView::addItem( QString searchreply )
{
	searchreply = searchreply.stripWhiteSpace();
	
	QString recievedSearchID = giFTConnection::searchKey( "ITEM", searchreply );
	
	if (recievedSearchID.toInt() == _ID )
	{
		if ( searchreply.find("file(") > -1 )
		{
			/*
			 * See if there is already a ListViewItem with this hash
			 * Make a child if there is
			 *
			 */
			
			QString hash = giFTConnection::searchKey("hash", searchreply);

			if ( hash=="" || hashes2items.find(hash) == 0 )
			{
				// We have a new unique hash make a parent and a child
				hashes2items.insert( hash, new ApollonSearchViewItem( searchresults ) );
				
				QString temp  = giFTConnection::searchKey("file", searchreply);
				KIconLoader *loader = KGlobal::iconLoader();
				QString protocol_name  = giFTConnection::searchKey("url", searchreply);
				protocol_name = protocol_name.left(protocol_name.find(":"));

				if(protocol_name=="FastTrack")
				{
					hashes2items[hash]->setPixmap(0, loader->loadIcon("kazaa", KIcon::Small));
				}
				else if(protocol_name=="OpenNap")
				{
					hashes2items[hash]->setPixmap(0, loader->loadIcon("napster", KIcon::Small));
				}
				else if(protocol_name=="Gnutella")
				{
					hashes2items[hash]->setPixmap(0, loader->loadIcon("gnutella", KIcon::Small));
				}
				else if(protocol_name=="OpenFT")
				{
					hashes2items[hash]->setPixmap(0, loader->loadIcon("openft", KIcon::Small));
				}
				hashes2items[hash]->setUrl(protocol_name);
				hashes2items[hash]->setFileName( temp.right(temp.length() - temp.findRev("/") - 1) );
				hashes2items[hash]->setSize( giFTConnection::searchKey("size", searchreply) );
				hashes2items[hash]->setHash( giFTConnection::searchKey("hash", searchreply) );
				hashes2items[hash]->setAlbum( giFTConnection::searchKey("album", searchreply) );
				hashes2items[hash]->setDuration( giFTConnection::searchKey("duration", searchreply) );
				hashes2items[hash]->setBitrate( giFTConnection::searchKey("bitrate", searchreply) );

				//Child

				hashes2items[hash]->addSource( searchreply );

				if(searchresults->childCount() == 1)
				{
				  //parent->changeTab(this, query + i18n(" (1 result)"));
					label->setText( labeltext_search + i18n(" (1 result)"));
				}
				else
				{
				  //parent->changeTab(this, query + i18n(" (%1 results)").arg(searchresults->childCount()));
				  label->setText( labeltext_search + i18n(" (%1 results)").arg(searchresults->childCount()) );
				}
			}
			else
			{

				// Not an unique item make a child
				hashes2items[hash]->addSource( searchreply );
			}
		}
		else
		{
			QString endsearch = "ITEM(" + QString::number( _ID ) + ");";

			if ( QString::compare( endsearch, searchreply) == 0 )
			{
			         label->setText( labeltext_complete + i18n(" (%1 results)").arg(searchresults->childCount()));
				 parent->changeTab(this, query + i18n(" (finished)"));
				 more->setEnabled(1);
				 stop->setEnabled(0);
			}
			else
			{
				//cout << searchreply << endl;
			}
		}
	}
}

void ApollonQueryView::browse()
{
	emit giftBrowseRequest( current );
}

void ApollonQueryView::browse(int index)
{
	current = browseusermenu->text(index);
	
	emit giftBrowseRequest( current );
}

