/***************************************************************************
                          apollonsearchtab.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
using namespace std;

#include <kglobal.h>
#include <kiconloader.h> 
#include <qsplitter.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qpixmap.h>
#include <qvbox.h>

#include <klocale.h>

#include "apollonsearchtab.h"
#include "giftconnection.h"

#define MAX_TABTEXT_CHARS 20

ApollonSearchTab::ApollonSearchTab(QWidget* parent, const char *name, giFTConnection* gcn): ApollonTab(parent, name, gcn)
{
  
        kqv = static_cast<ApollonQueryView*>(0);
	KIconLoader *loader = KGlobal::iconLoader();
	
	QVBox* splitter = new QVBox(this);
	
	QWidget* searchpanel = new QWidget(splitter);
	QGridLayout *grid = new QGridLayout( searchpanel, 2, 3 );

	QLabel* querylabel = new QLabel(searchpanel);
	querylabel->setText(i18n("Search for "));
	querylabel->setFixedHeight( 20 );

	QLabel* keylabel =   new QLabel(searchpanel);
	keylabel->setText(i18n("with keywords: "));
	keylabel->setFixedHeight( 20 );

	querybox = new QLineEdit(searchpanel);
	
	// This is a default text to make hacking easier
	//querybox->setText("MTV britney");

	querycombo = new QComboBox(searchpanel);

	querycombo->insertItem(loader->loadIcon("find",KIcon::Small) , "everything", 0 );
	querycombo->insertItem(loader->loadIcon("sound",KIcon::Small), "audio", 1 );
	querycombo->insertItem(loader->loadIcon("video",KIcon::Small), "video", 2 );
	querycombo->insertItem(loader->loadIcon("image",KIcon::Small), "images", 3 );
	querycombo->insertItem(loader->loadIcon("txt",KIcon::Small), "text documents", 4 );
	querycombo->insertItem(loader->loadIcon("binary",KIcon::Small), "software", 5 );
	querycombo->insertItem(loader->loadIcon("connect_established",KIcon::Small), "user", 6 );
	/*
	querycombo->insertItem(loader->loadIcon("help_index",KIcon::Small), "hash", 7 );
	*/
	QPushButton* go = new QPushButton(searchpanel);
	go->setPixmap( loader->loadIcon("down", KIcon::Toolbar));
	
	grid->addWidget(querylabel, 0, 0);
	grid->addWidget(querycombo, 0, 1);
	grid->addWidget(keylabel, 1, 0);
	grid->addWidget(querybox, 1, 1);
	grid->addWidget(go, 1, 2);

	grid->setRowStretch(0, 0);
	grid->setRowStretch(1, 0);
	grid->setColStretch(0, 0);
	grid->setColStretch(1, 1);
	grid->setColStretch(2, 0);

	viewtab = new QTabWidget(splitter);
	
	viewtab->setTabPosition(QTabWidget::Bottom);
	
	connect(viewtab, SIGNAL(currentChanged(QWidget*)), this, SLOT(sltChangeTab(QWidget*)));

	connect( go, SIGNAL(clicked()),			this, SLOT(startSearch()) );
	connect( querybox, SIGNAL(returnPressed()), 	this, SLOT(startSearch()) );
	
	connect(_gcn, SIGNAL(giftItemReply(QString)),	this, SIGNAL(giftItemReply(QString)));
	
}


ApollonSearchTab::~ApollonSearchTab() 
{
  if (kqv)
    delete kqv;
}

void ApollonSearchTab::startQuery( const int& index, const QString& text )
{
	KIconLoader *loader = KGlobal::iconLoader();

	int ID = _gcn->getNewID();

	_gcn->makeSearchRequest( ID, index, text );

	if( index == 6 )
	{
		BrowseUser(text);
		return;
	}
	else
	{
		kqv = new ApollonQueryView( viewtab, ID, index, text, _gcn);
	}
	
	QString tabtext =  text;
	tabtext = (tabtext.length() < MAX_TABTEXT_CHARS ) ? tabtext : tabtext.left(MAX_TABTEXT_CHARS - 3).append("...") ;

	QPixmap icon;

	switch (index)
	{
		case 0:
			icon = loader->loadIcon("find",KIcon::Small);
			break;
		case 1:
			icon = loader->loadIcon("sound",KIcon::Small);
			break;
		case 2:
			icon = loader->loadIcon("video",KIcon::Small);
			break;
		case 3:
			icon = loader->loadIcon("image",KIcon::Small);
			break;
		case 4:
			icon = loader->loadIcon("txt",KIcon::Small);
			break;
		case 5:
			icon = loader->loadIcon("binary",KIcon::Small);
			break;
		/*
		case 6:
			icon = loader->loadIcon("connect_established",KIcon::Small);
			break;
		case 7:
			icon = loader->loadIcon("help_index",KIcon::Small);
			break;
		*/
			
	}
	
	//	viewtab->addTab(kqv, icon, tabtext + i18n(" (%1 results)").arg(0) );
	viewtab->addTab(kqv, icon, tabtext + i18n(" (searching...)" ));
	
	viewtab->showPage(kqv);

	connect( this, SIGNAL(giftItemReply(QString)), kqv, SLOT(addItem(QString)) );
	connect( kqv, SIGNAL( giftAddRequest( QString, QString, QString, QString, QString ) ), _gcn, SLOT( makeAddRequest( QString, QString, QString, QString, QString ) ) );
	connect( kqv, SIGNAL( giftCancelSearchRequest(int) ), _gcn, SLOT( makeCancelSearchRequest(int) ) );
	connect( kqv, SIGNAL( giftBrowseRequest(QString) ), this, SLOT( BrowseUser(QString) ) );
}

void ApollonSearchTab::startSearch()
{
	// find out which realm thing is selected and what text is searched for
	int index = querycombo->currentItem();
	QString text = querybox->text();

	startQuery( index, text );
}

void ApollonSearchTab::BrowseUser(QString string)
{
	KIconLoader *loader = KGlobal::iconLoader();

	int ID = _gcn->getNewID();

	_gcn->makeBrowseRequest( ID, string );

	kbv = new ApollonBrowseView( viewtab, ID, string );

	QString tabtext =  string;
	tabtext = (tabtext.length() < MAX_TABTEXT_CHARS ) ? tabtext : tabtext.left(MAX_TABTEXT_CHARS - 3).append("...") ;

	QPixmap icon;
	icon = loader->loadIcon("connect_established",KIcon::Small);

	viewtab->addTab(kbv, icon, tabtext);
	viewtab->showPage(kbv);

	connect( this, SIGNAL(giftItemReply(QString)), kbv, SLOT(addItem(QString)) );
	connect( kbv, SIGNAL( giftAddRequest( QString, QString, QString, QString, QString ) ), _gcn, SLOT( makeAddRequest( QString, QString, QString, QString, QString ) ) );
	connect( kbv, SIGNAL( giftCancelBrowseRequest(int) ), _gcn, SLOT( makeCancelBrowseRequest(int) ) );
}


void ApollonSearchTab::sltChangeTab(QWidget* w) {
  ApollonQueryView* k = dynamic_cast<ApollonQueryView*>(w);
  ApollonBrowseView* b = dynamic_cast<ApollonBrowseView*>(w);
  if (k) {
    querybox->setText(k->getQuery());
    querycombo->setCurrentItem(k->getIndex());
  }
  if (b) {
    querybox->setText(b->getQuery());
    querycombo->setCurrentItem(b->getIndex());
  }
}
