/***************************************************************************
                          apollonsearchitemview.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
using namespace std;

#include <qstring.h>
#include <qregexp.h>

#include <klocale.h>

#include "apollonsearchviewitem.h"
#include "apollonsearchviewitemchild.h"
#include "apollonutils.h"
#include "giftconnection.h"

ApollonSearchViewItem::ApollonSearchViewItem( QListView * parent ) : QListViewItem( parent )
{
}

ApollonSearchViewItem::ApollonSearchViewItem( QListViewItem * parent ) : QListViewItem( parent )
{
}

ApollonSearchViewItem::~ApollonSearchViewItem()
{
}

QString ApollonSearchViewItem::text(int col) const
{
	QString line;

	switch(col)
	{
		case 1:
			line = file;
			break;
		case 2:
			line = ApollonUtils::convert_size(size.toULong());
			break;
		case 3:
			line = QString::number(children.count()) + i18n(" source(s)");
			break;
		case 4:
			line = user;
			break;
		case 5:
			line = album;
			break;
		case 6:
			if(!duration.isEmpty())
			{
				line = ApollonUtils::timeHuman(duration.toULong());
			}
			break;
		case 7:
			if(!bitrate.isEmpty())
			{
				line = ApollonUtils::bitrateHuman(bitrate.toULong());
			}
			break;
	}

	return line;
}

QString ApollonSearchViewItem::key(int column, bool /*ascending*/) const
{
	switch(column)
	{
		case 1:
			return text(column).lower();
			break;
		case 0:
			return url;
		case 2:
			return size.rightJustify(15, '0');
			break;
		case 3:
		        return QString::number(children.count());
			break;
		case 6:
			return duration;
			break;
		case 7:
			return bitrate;
			break;
		default:
			return text(column);
			break;
	}
}
int ApollonSearchViewItem::compare( QListViewItem* item, int column, bool ascending ) const
{
	if ( column == 3 || column == 6 || column == 7  ) {
	QString current = key( column, ascending);
	QString next = item->key( column, ascending);
	return current.toULong() - next.toULong();
	} else {
	return QListViewItem::compare( item, column, ascending );
	}
	}


void  ApollonSearchViewItem::addSource ( QString searchreply )
{
	QString user = giFTConnection::searchKey("user", searchreply );

	if ( children.find(user) == 0 )
	{
		// This is a unique child ass it to the listviewitem
		
		children.insert( user, new ApollonSearchViewItemChild( this ) );

		QString temp  = giFTConnection::searchKey("file", searchreply);
		children[user]->setFileName( temp.right(temp.length() - temp.findRev("/") - 1) );
		children[user]->setSize( giFTConnection::searchKey("size", searchreply) );

		children[user]->setUser( user );
		children[user]->setHash( giFTConnection::searchKey("hash", searchreply) );
		children[user]->setUrl( giFTConnection::searchKey("url", searchreply) );

		children[user]->setOpenSlots( giFTConnection::searchKey( "availability", searchreply) );

                children[user]->setAlbum(giFTConnection::searchKey("album", searchreply));
		children[user]->setDuration( giFTConnection::searchKey("duration", searchreply));
		children[user]->setBitrate( giFTConnection::searchKey("bitrate", searchreply));
	}
	
}


