/***************************************************************************
                          Apollontransfertab.cpp  -  description
                             -------------------
    begin                : Tue Apr 2 2002
    copyright            : (C) 2002 by Robert Wittams
    email                : robert@wittams.com en Je moeder
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include <assert.h>

#include <qdir.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qvbox.h>

#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kpassivepopup.h>
#include <kconfig.h>

#include "giftconnection.h"
#include "apollonplayertab.h"
#include "apollontransfertab.h"
#include "apollontransferviewitem.h"
#include "apollonutils.h"
#include "preferences.h"


ApollonTransferTab::ApollonTransferTab(QWidget* parent, const char *name, giFTConnection* gcn) :
  ApollonTab(parent, name, gcn),
  m_playerTab(0)
{
	KIconLoader *loader = KGlobal::iconLoader();

	connect(_gcn, SIGNAL(giftAddDownloadReply(QString)), this, SLOT(addDownloadViewItem(QString)));
	connect(_gcn, SIGNAL(giftAddUploadReply(QString)),   this, SLOT(addUploadViewItem(QString)));
	connect(_gcn, SIGNAL(giftChgUploadReply(QString)),   this, SLOT(updateUploadView(QString)));
	connect(_gcn, SIGNAL(giftChgDownloadReply(QString)), this, SLOT(updateDownloadView(QString)));
	connect(_gcn, SIGNAL(giftDelDownloadReply(QString)), this, SLOT(delDownloadViewItem(QString)));
	connect(_gcn, SIGNAL(giftDelUploadReply(QString)),   this, SLOT(delUploadViewItem(QString)));
	connect(_gcn, SIGNAL(giftAddSourceReply(QString)),   this, SLOT(addSourceItem(QString)));
	connect(_gcn, SIGNAL(giftItemReply(QString)),        this, SLOT(ItemReply(QString)));
	connect(_gcn, SIGNAL(disconnected()),                this, SLOT(disconnected()));

	QSplitter* splitter = new QSplitter(this);
	splitter->setOrientation(QSplitter::Vertical);

	setMinimumWidth(400);
	QVBox* downbox = new QVBox(splitter);

	new QLabel(i18n("Downloads"), downbox);
	downlist = new KListView(downbox);

	downlist->setSelectionMode(QListView::Extended);

	downlist->addColumn( i18n("Name") );
	downlist->setColumnWidthMode(0, QListView::Manual);
	downlist->setColumnWidth(0, 400);

	downlist->addColumn( i18n("Size") );
	downlist->setColumnAlignment(1, Qt::AlignRight);

	downlist->addColumn( i18n("Transferred") );
	downlist->setColumnAlignment(2, Qt::AlignRight);

	downlist->addColumn( i18n("Bandwidth") );
	downlist->setColumnAlignment(3, Qt::AlignRight);

	downlist->addColumn( i18n("Progress") );
	downlist->setColumnAlignment(4, Qt::AlignRight);

	downlist->addColumn( i18n("ETA") );
	downlist->setColumnAlignment(5, Qt::AlignRight);

	downlist->setAllColumnsShowFocus(TRUE);
	downlist->setRootIsDecorated(TRUE);
	downlist->setShowSortIndicator(TRUE);

	downmenu = new QPopupMenu(downlist);
	downmenu->insertItem(loader->loadIcon("player_play",   KIcon::Small), i18n("&Preview"),                 this, SLOT(previewFile()),    Key_Return, 0, 0);
	downmenu->insertItem(loader->loadIcon("connect_no",   KIcon::Small), i18n("P&ause"),                 this, SLOT(pauseDownload()),    Key_P, 1, 1);
	downmenu->insertItem(loader->loadIcon("viewmag+",      KIcon::Small), i18n("&Find more sources"),       this, SLOT(sourceSearch()),   Key_Plus, 2, 2);
	downmenu->insertItem(loader->loadIcon("stop",          KIcon::Small), i18n("&Cancel Download"),         this, SLOT(cancelDownload()), Key_Delete, 3, 3);
	downmenu->insertSeparator();
	downmenu->insertItem(loader->loadIcon("trashcan_full", KIcon::Small), i18n("C&lear Finished/Canceled"), this, SLOT(clearFinishedDownloads()));

	browseusermenu = new QPopupMenu(downlist);

	connect(downlist, SIGNAL(rightButtonPressed( QListViewItem*, const QPoint&, int)), this, SLOT(downloadPopup(QListViewItem*, const QPoint&, int)));
	connect(downlist, SIGNAL(expanded(QListViewItem *)), this, SLOT(updateDownloadSources(QListViewItem *)));

	QVBox* upbox = new QVBox(splitter);

	new QLabel(i18n("Uploads"), upbox);
	uplist = new KListView(upbox);
	uplist->addColumn( i18n("Name") );
	uplist->setColumnWidthMode(0, QListView::Manual);
	uplist->setColumnWidth(0, 200);
	uplist->addColumn( i18n("Size") );
	uplist->setColumnAlignment(1, Qt::AlignRight);

	uplist->addColumn( i18n("Transferred") );
	uplist->setColumnAlignment(2, Qt::AlignRight);

	uplist->addColumn( i18n("Bandwidth") );
	uplist->setColumnAlignment(3, Qt::AlignRight);

	uplist->addColumn( i18n("Progress") );
	uplist->setColumnAlignment(4, Qt::AlignRight);

	uplist->addColumn( i18n("ETA") );
	uplist->setColumnAlignment(5, Qt::AlignRight);

	uplist->setAllColumnsShowFocus(TRUE);
	uplist->setRootIsDecorated(TRUE);
	uplist->setShowSortIndicator(TRUE);

	upmenu = new QPopupMenu(uplist);
	upmenu->insertItem(loader->loadIcon("stop",          KIcon::Small), i18n("&Cancel Upload"),           this, SLOT(cancelUpload()),   Key_Delete);
	upmenu->insertSeparator();
	upmenu->insertItem(loader->loadIcon("trashcan_full", KIcon::Small), i18n("C&lear Finished/Canceled"), this, SLOT(clearFinishedUploads()));

	connect(uplist, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(uploadPopup(QListViewItem *, const QPoint &, int)));
	connect(uplist, SIGNAL(expanded(QListViewItem *)), this, SLOT(updateUploadSources(QListViewItem *)));

	splitter->setResizeMode( upbox, QSplitter::KeepSize);
	splitter->show();
	showPopup = true;
	
	KConfig* config = KGlobal::config();
	downlist->restoreLayout (config, "TransferTabDown");
	uplist->restoreLayout (config, "TransferTabUp");
	
}


ApollonTransferTab::~ApollonTransferTab()
{
  KConfig* config = KGlobal::config();
  downlist->saveLayout (config, "TransferTabDown");
  uplist->saveLayout(config, "TransferTabUp");
  config->sync();
}


void ApollonTransferTab::addDownloadViewItem(QString line)
{
	int     ID =       giFTConnection::searchKey("ADDDOWNLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	QString hash =     giFTConnection::searchKey("hash", line);
	QString file =     giFTConnection::searchKey("file", line);
	QString  protocol_name = giFTConnection::searchKey("url", line);
	protocol_name = protocol_name.left(protocol_name.find(":"));

	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = new ApollonTransferViewItem(downlist, "download", ID, protocol_name);

	item->setHash(hash);
	item->setName(file);
	item->setSize(size);
	item->setTransmit(transmit);
	item->setStatus(QString("%1%").arg(progress, 0, 'f', 1));
	item->setBandwidth("--");
	
	int numberofsources = line.contains("SOURCE {", true);

	for(int i = 0; i < numberofsources; i++)
	{
		QString source = line.section("SOURCE {", i + 1, i + 1);

		ApollonTransferViewItem *sourceitem = new ApollonTransferViewItem(item);

		sourceitem->setName(giFTConnection::searchKey("user", source));

		sourceitem->setTransmit(giFTConnection::searchKey("transmit", source).toInt());

		sourceitem->setStatus(giFTConnection::searchKey("status", source));
	}
}

void ApollonTransferTab::delDownloadViewItem(QString line)
{
	int ID = giFTConnection::searchKey("DELDOWNLOAD", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(downlist, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid delDownloadView request." << endl;
		return;
	}

	if(item->size() == item->transmit())
	{
		item->setStatus(i18n("Completed"));

		if(showPopup)
		{
			KPassivePopup::message(i18n("Download of %1 finished").arg(item->name()), systemTray);
		}
	}
	else
	{
		item->setStatus(i18n("Canceled"));
	}
}

void ApollonTransferTab::addUploadViewItem(QString line)
{
	int     ID =       giFTConnection::searchKey("ADDUPLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	QString file =     giFTConnection::searchKey("file", line);
	QString  protocol_name = giFTConnection::searchKey("url", line);
	protocol_name = protocol_name.left(protocol_name.find(":"));
	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = new ApollonTransferViewItem(uplist, "upload", ID, protocol_name);

	item->setName(file);
	item->setSize(size);
	item->setTransmit(transmit);
	item->setStatus(QString("%1 %").arg(progress, 0, 'f', 1) );
	int numberofsources = line.contains("SOURCE {", true);

	for(int i = 0; i < numberofsources; i++)
	{
		QString source = line.section("SOURCE {", i + 1, i + 1);

		ApollonTransferViewItem *sourceitem = new ApollonTransferViewItem(item);

		sourceitem->setName(giFTConnection::searchKey("user", source));
		sourceitem->setSize(giFTConnection::searchKey("transmit", source).toInt());
		sourceitem->setStatus(giFTConnection::searchKey("status", source));
	}
}

void ApollonTransferTab::delUploadViewItem(QString line)
{
	int ID = giFTConnection::searchKey("DELUPLOAD", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(uplist, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid delUploadViewItem request." << endl;
		return;
	}

	if(item->size() == item->transmit())
	{
		item->setStatus(i18n("Completed"));
	}
	else
	{
		item->setStatus(i18n("Canceled"));
	}
}


void ApollonTransferTab::updateDownloadView(QString line)
{
	int     ID =        giFTConnection::searchKey("CHGDOWNLOAD", line).toInt();
	int     transmit =  giFTConnection::searchKey("transmit", line).toInt();
	int     through =   giFTConnection::searchKey("throughput", line).toInt();
	int     elapsed =   giFTConnection::searchKey("elapsed", line).toInt();
	int     size =      giFTConnection::searchKey("size", line).toInt();
	QString bandwidth = ApollonUtils::calcBandwith(through, elapsed);
	QString  protocol_name = giFTConnection::searchKey("url", line);
	protocol_name = protocol_name.left(protocol_name.find(":"));
	float   progress =  100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(downlist, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid updateDownloadView request." << endl;
		return;
	}
	item->updateIcon(protocol_name, "download");
	item->setTransmit(transmit);
	item->setBandwidth(bandwidth);
	item->setStatus(QString("%1 %").arg(progress, 0, 'f', 1));
	item->setETA(through == 0 ? -1 : (int) ((size - transmit) / 1000 * ((float) elapsed / through)));

	item->setUpdateString(line);

	if(item->isOpen())
	{
		updateDownloadSources(item);
	}

	countActives();
}

void ApollonTransferTab::updateDownloadSources(QListViewItem *item)
{
	QString line = dynamic_cast<ApollonTransferViewItem *>(item)->updateString();

	int     elapsed =   giFTConnection::searchKey("elapsed", line).toInt();

	QString source = line.section("SOURCE {", 0, 0);
	line.remove(0, source.length());

	ApollonTransferViewItem *sourceItem = dynamic_cast<ApollonTransferViewItem *>(item)->firstChild();

	while(source = line.section("SOURCE {", 1, 1), !source.isEmpty())
	{
		if(sourceItem == 0)
		{
			sourceItem = new ApollonTransferViewItem(item);
		}

		QString sourceName =   giFTConnection::searchKey("user", source);
		QString sourceStatus = giFTConnection::searchKey("status", source);

		sourceItem->setName(sourceName);
		sourceItem->setStatus(sourceStatus);

		if(sourceStatus == i18n("Active"))
		{
			int sourceTransmit = giFTConnection::searchKey("transmit", source).toInt();
			int aux = sourceTransmit - sourceItem->transmit();

			sourceItem->setTransmit(sourceTransmit);
			sourceItem->setBandwidth(ApollonUtils::calcBandwith(aux, elapsed));
		}
		else
		{
			sourceItem->setTransmit(-1);
			sourceItem->setBandwidth("");
		}

		sourceItem = sourceItem->nextSibling();

		line.remove(0, source.length() + 8);
	}
}

void ApollonTransferTab::updateUploadView(QString line)
{
	int     ID =       giFTConnection::searchKey("CHGUPLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     through =  giFTConnection::searchKey("throughput", line).toInt();
	int     elapsed =  giFTConnection::searchKey("elapsed", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(uplist, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid updateUploadView request." << endl;
		return;
	}

	item->setTransmit(transmit);
	item->setBandwidth(ApollonUtils::calcBandwith(through, elapsed));
	item->setStatus(QString("%1 %").arg(progress, 0, 'f', 1));
	item->setETA(through == 0 ? -1 : ((int) ((size - transmit) / 1000 * ((float) elapsed / through))));

	item->setUpdateString(line);

	if(item->isOpen())
	{
		updateUploadSources(item);
	}
}

void ApollonTransferTab::updateUploadSources(QListViewItem *item)
{
	QString line = dynamic_cast<ApollonTransferViewItem *>(item)->updateString();

	QString source = line.section("SOURCE {", 0, 0);
	line.remove(0, source.length());

	ApollonTransferViewItem *sourceItem = dynamic_cast<ApollonTransferViewItem *>(item)->firstChild();

	while(source = line.section("SOURCE {", 1, 1), !source.isEmpty())
	{
   		if(sourceItem == 0)
		{
			sourceItem = new ApollonTransferViewItem(item);
		}

		QString sourceName =   giFTConnection::searchKey("user", source);
		QString sourceStatus = giFTConnection::searchKey("status", source);

		sourceItem->setName(sourceName);
		sourceItem->setStatus(sourceStatus);

		if(sourceStatus == i18n("Active"))
		{
			int sourceTransmit = giFTConnection::searchKey("transmit", source).toInt();

			sourceItem->setTransmit(sourceTransmit);
		}
		else
		{
			sourceItem->setTransmit(-1);
		}

		sourceItem = sourceItem->nextSibling();

		line.remove(0, source.length() + 8);
	}
}

void ApollonTransferTab::addSourceItem(QString line)
{
	int ID = giFTConnection::searchKey("ADDSOURCE", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(downlist, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid addSourceItem request." << endl;
		return;
	}

	ApollonTransferViewItem *sourceItem = new ApollonTransferViewItem(item);

	sourceItem->setName(giFTConnection::searchKey("user", line));
	sourceItem->setStatus(giFTConnection::searchKey("status", line));
}

void ApollonTransferTab::disconnected()
{
	clearAll();
}

void ApollonTransferTab::downloadPopup(QListViewItem *item, const QPoint &point, int)
{
	current = 0;

	if(!item)
	{
		return;
	}

	KIconLoader *loader = KGlobal::iconLoader();

	ApollonTransferViewItem* transview = (ApollonTransferViewItem*) item;

	browseusermenu->clear();

	if (transview->parent() == 0)
	{
		current = transview->id();
		
		if (transview->isPaused()) {
		  downmenu->removeItem(1);

		  downmenu->insertItem(loader->loadIcon("connect_established",   KIcon::Small), i18n("&Resume"),                 this, SLOT(unpauseDownload()),    Key_R, 1,1);
		}
		else {
		  downmenu->removeItem(1);
		  downmenu->insertItem(loader->loadIcon("connect_no",   KIcon::Small), i18n("P&ause"),                 this, SLOT(pauseDownload()),    Key_P, 1,1);
		}
		
		downmenu->popup(point);
	}
	else
	{
		current = transview->parent()->id();
		userIP =  transview->name();
		browseusermenu->insertItem(loader->loadIcon("find", KIcon::Small), i18n("Browse user: ") + userIP, this, SLOT(sltBrowseUser()));
		browseusermenu->popup(point);
	}
}

void ApollonTransferTab::uploadPopup(QListViewItem *item, const QPoint &point, int)
{
	current = 0;

	if(!item)
	{
		return;
	}

	ApollonTransferViewItem *transview = (ApollonTransferViewItem*) item;

	if(transview->parent() == 0)
	{
		current = transview->id();
	}
	else
	{
		current = transview->parent()->id();
	}

	upmenu->popup(point);
}


void ApollonTransferTab::cancelDownload()
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) downlist->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		{
			if(item->parent() == 0)
			{
				current = item->id();
			}
			else
			{
				current = item->parent()->id();
				item = item->parent();
			}
			_gcn->makeTransferRequest(current, "cancel");
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::pauseDownload()
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) downlist->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		{
			if(item->parent() == 0)
			{
				current = item->id();
				item->pauseItem();
			}
			else
			{
				current = item->parent()->id();
				item = item->parent();
			}
			_gcn->makeTransferRequest(current, "pause");
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::unpauseDownload()
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) downlist->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		{
			if(item->parent() == 0)
			{
				current = item->id();
				item->unpauseItem();
			}
			else
			{
				current = item->parent()->id();
				item = item->parent();
			}
			_gcn->makeTransferRequest(current, "unpause");
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::cancelUpload()
{
	_gcn->makeTransferRequest(current, "cancel");
}

void ApollonTransferTab::clearFinishedDownloads()
{
	ApollonTransferViewItem *item = (ApollonTransferViewItem *) downlist->firstChild();

	clearFinished(item);
}

void ApollonTransferTab::clearFinishedUploads()
{
	ApollonTransferViewItem *item = (ApollonTransferViewItem *) uplist->firstChild();

	clearFinished(item);
}

void ApollonTransferTab::clearFinished(ApollonTransferViewItem *item)
{
	ApollonTransferViewItem *temp;

	while(item)
	{
		temp = item->nextSibling();
		if(item->status() == i18n("Canceled") || item->status() == i18n("Completed"))
		{
			delete item;
		}
		item = temp;
	}
}

void ApollonTransferTab::clearAll()
{
	ApollonTransferViewItem *p = (ApollonTransferViewItem *) downlist->firstChild(), *q;
	while(p)
	{
		q = p->nextSibling();
		delete p;
		p = q;
	}

	p = (ApollonTransferViewItem *) uplist->firstChild();
	while(p)
	{
		q = p->nextSibling();
		delete p;
		p = q;
	}
}

void ApollonTransferTab::previewFile()
{
	assert(m_playerTab != 0);
	assert(m_preferences != 0);

	QString url;

	if(((ApollonTransferViewItem *) downlist->currentItem())->status() == i18n("Completed"))
	{
		url = QString("file:") + m_preferences->getCompletedPath() + "/" + ((ApollonTransferViewItem *) downlist->currentItem())->name();
		m_playerTab->play(url);
	}
	else
	{
		QDir dir(m_preferences->getIncomingPath());
		url = QString("file:") + dir.path() + "/" + dir.entryList().grep(((ApollonTransferViewItem *) downlist->currentItem())->name())[0];
		m_playerTab->play(url, ((ApollonTransferViewItem *) downlist->currentItem())->name());
	}
}

void ApollonTransferTab::autoSourceSearch()
{
	sourceSearch(true);
}

void ApollonTransferTab::sourceSearch(bool autoSearch)
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem*) downlist->firstChild();
	ApollonTransferViewItem* sourcesearchitem;
	while(item)
	{
		sourcesearchitem = item;
		if((autoSearch || sourcesearchitem->isSelected()) &&
		   sourcesearchitem->status() != i18n("Canceled") && sourcesearchitem->status() != i18n("Completed"))
		{
			int ID = _gcn->getNewID();

			sourcesearchids.append(ID);

			_gcn->makeLocateRequest(ID, sourcesearchitem->hash());
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::ItemReply(QString line)
{
	line = line.stripWhiteSpace();

	int recievedItemID = giFTConnection::searchKey("ITEM", line).toInt();

	if(sourcesearchids.contains(recievedItemID) > 0)
	{
		// Should there be a test here to see if this reply is already in the list ?

		if ( line.find("file(") > -1 )
		{
			QString user = giFTConnection::searchKey("user", line);
			QString hash = giFTConnection::searchKey("hash", line);
			QString size = giFTConnection::searchKey("size", line);
			QString url =  giFTConnection::searchKey("url",  line);

			QString temp = giFTConnection::searchKey("file", line);
			QString save = temp.right(temp.length() - temp.findRev("/") - 1);

			_gcn->makeAddRequest( user, hash, size, url, save );
		}
		else
		{
			QString endsearch = "ITEM(" + QString::number(recievedItemID) + ");";

			if ( QString::compare( endsearch, line) == 0 )
			{
				sourcesearchids.remove(recievedItemID);
			}
		}
	}
}

void ApollonTransferTab::sltBrowseUser()
{
	emit sigBrowseUser(userIP);
}

void ApollonTransferTab::sltGetUserIP(int id)
{
	userIP = browseusermenu->text(id);
}

void ApollonTransferTab::countActives()
{
	// sends a signal to the systemtray, which displays the number of downloads in the QToolTip.
	int actives = 0;
	float sumrate = 0;
	QString s;
	ApollonTransferViewItem* item = (ApollonTransferViewItem*) downlist->firstChild();
	ApollonTransferViewItem* tvitem;
	while(item)
	{
		tvitem = item->firstChild();
		while(tvitem)
		{
			if(tvitem->status() == i18n("Active"))
			{
				actives++;
				s = tvitem->bandwidth();
				sumrate += (s.remove(" KB/s")).toFloat();
			}
			tvitem = tvitem->nextSibling();
		}

		item = item->nextSibling();
	}
	emit sigSendInfo(actives, sumrate);
}

void ApollonTransferTab::setTray(ApollonSystemTray* tray) {
	systemTray = tray;
}

void ApollonTransferTab::setShowPopup(bool show) {
	showPopup = show;
}


void ApollonTransferTab::sltAutoSearchInterval(int i)
{
	m_autoSearchInterval = i;
	m_sourceSearchTimer = new QTimer;
	m_sourceSearchTimer->start(m_autoSearchInterval * 60000); // m_autoSearchInterval is in minutes
	connect(m_sourceSearchTimer, SIGNAL(timeout()), this, SLOT(autoSourceSearch()));
}
