/***************************************************************************
                          apollontransferviewitem.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <iostream>
using namespace std;

#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>

#include "apollontransferviewitem.h"

ApollonTransferViewItem::ApollonTransferViewItem(QListView *parent, const QString &updown, int ID, const QString &protocol_name) :
  QListViewItem(parent),
  m_ID(ID),
  m_hash(QString::null),
  m_name(QString::null),
  m_size(-1),
  m_transmit(-1),
  m_bandwidth(QString::null),
  m_status(QString::null),
  m_eta(-1),
  m_updateString(QString::null)
{
  m_Updated = FALSE;
  updateIcon(protocol_name, updown);
  m_Updated = FALSE;
  m_isPaused = FALSE;
  KIconLoader *loader = KGlobal::iconLoader();
  m_PausePix = loader->loadIcon("player_pause", KIcon::Small);

}

ApollonTransferViewItem::ApollonTransferViewItem(QListViewItem *parent, int ID) :
  QListViewItem(parent),
  m_ID(ID),
  m_hash(QString::null),
  m_name(QString::null),
  m_size(-1),
  m_transmit(-1),
  m_bandwidth(QString::null),
  m_status(QString::null),
  m_eta(-1),
  m_updateString(QString::null)
{
  m_Updated = FALSE;
  m_isPaused = FALSE;
  KIconLoader *loader = KGlobal::iconLoader();
  m_PausePix = loader->loadIcon("player_pause", KIcon::Small);
}

ApollonTransferViewItem::~ApollonTransferViewItem()
{
}

void ApollonTransferViewItem::setStatus(QString status)
{
	m_status = status;

	KIconLoader *loader = KGlobal::iconLoader();

	if(m_status == i18n("Completed"))
	{
		setPixmap(0, loader->loadIcon("ok", KIcon::Small));
		m_bandwidth = "";
		m_eta = -1;
	}
	else if(m_status == i18n("Canceled"))
	{
		setPixmap(0, loader->loadIcon("no", KIcon::Small));
		m_bandwidth = "";
		m_eta = -1;
	}
	setText(columnStatus, m_status);
}

ApollonTransferViewItem *ApollonTransferViewItem::getItem(const QListView *lv, int ID)
{
	assert(lv != 0);

	ApollonTransferViewItem *item = (ApollonTransferViewItem*) lv->firstChild();
	while(item != 0)
	{
		if(item->id() == ID)
		{
			break;
		}
		item = item->nextSibling();
	}

	return item;
}


void ApollonTransferViewItem::updateIcon(const QString & protocol_name, const QString & updown) {

  if (! m_Updated) {
  	KIconLoader *loader = KGlobal::iconLoader();

	if(updown == "download")
	{
	         if(protocol_name=="FastTrack")
		  {
		    mIcon = "kazaadown";
		  }
		else if(protocol_name=="OpenNap")
		  {
		    mIcon = "napsterdown";
		  }
		else if(protocol_name=="Gnutella")
		  {
		    mIcon = "gnutelladown";
		  }
		else if(protocol_name=="OpenFT")
		  {
		    mIcon = "openftdown";
		  }
		 else
		   mIcon = "1downarrow";
	}
	
	if(updown == "upload")
	{
	        if(protocol_name=="FastTrack")
		  {
		    mIcon = "kazaaup";
		  }
		else if(protocol_name=="OpenNap")
		  {
		    mIcon = "napsterup";
		  }
		else if(protocol_name=="Gnutella")
		  {
		    mIcon = "gnutellaup";
		  }
		else if(protocol_name=="OpenFT")
		  {
		    mIcon = "openftup";
		  }
		else
		  mIcon = "1uparrow";
	}
	
	m_Pix = loader->loadIcon(mIcon, KIcon::Small);
	setPixmap(0,m_Pix);
	m_Updated = TRUE;

  }
}

void ApollonTransferViewItem::pauseItem() {
  setPixmap(0, m_PausePix);
  m_isPaused = TRUE;
  m_Updated = TRUE; //don't change the icon....
}

void ApollonTransferViewItem::unpauseItem() {
  setPixmap(0, m_Pix);
  m_isPaused = FALSE;
  m_Updated = TRUE;
}
QString ApollonTransferViewItem::key(int column, bool /*ascending*/) const
{
	QString temp1;
	switch(column)
	{
		case 1:
			temp1 = temp1.setNum(size());
			return temp1;
			break;
		case 2:
			temp1 = temp1.setNum(transmit());
			return temp1;
			break;
		case 3:
			temp1 = bandwidth();
			if(temp1=="--") return "0.00";
			temp1 = temp1.remove(" KB/s");
			return temp1;
			break;
		case 4:
			temp1 = status();
			temp1 = temp1.left(temp1.find(".")+2);
			return temp1;
			break;
		case 5:
			temp1 = temp1.setNum(eta());
			return temp1;
			break;
		default:
			return text(column);
			break;
	}
}
int ApollonTransferViewItem::compare( QListViewItem* item, int column, bool ascending ) const
{
	if ( column != 0  ) {
	QString current = key( column, ascending);
	QString next = item->key( column, ascending);
	if((current.toDouble() - next.toDouble())>0)
	return 1;
	else return -1;
	} else {
	return QListViewItem::compare( item, column, ascending );
	}
	}

