/***************************************************************************
                          giftconnection.h  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GIFTCONNECTION_H
#define GIFTCONNECTION_H

#include <qsocket.h>
#include <qstring.h>
#include <qptrqueue.h>
#include <sys/types.h>

class giFTConnection : public QObject  
{
  Q_OBJECT
public:
	giFTConnection();
	~giFTConnection();

	const QString& host() { return _host; }
	const int& port() { return _port; }
	bool connectToServer();
	bool isConnected();

	void setParams( QString host, int port );
	void disconnect();
	
	int getNewID();
	static QString searchKey(const QString &key, const QString &searchstr);
	void startgiFT();

	


public slots:
	void sockReadReady();
	void sockClosed();
	void sockError( int );
	
	void makeAttachRequest();
	void makeStatsRequest();
	void makeSearchRequest( const int ID, const int index, QString text );
	void makeCancelSearchRequest( int );
	void makeCancelBrowseRequest( int );
	void makeLocateRequest( const int ID, const QString hash );
	void makeAddRequest( QString user, QString hash, QString size, QString url, QString file );
	void makeTransferRequest( const int ID, const QString action );
	void makeShareRequest( QString action );
	void makeBrowseRequest( const int ID, QString user );
	void makeQuitRequest();
	
	void addRequest( QString& req );
	void sockWrite();
	
	QString escapeStr( QString& sendstr );
	void sltRestartDaemon();
	

private:
	QSocket socket;
	bool paramsSet;
	
	int ID;
	
	QString _host;
	int _port;
	pid_t giFT_pid;
	
	QPtrQueue<QString> pending;

signals:
	void connected();
	void disconnected();
	void socketError( int error );
	void giftStatusReply( QString );
	void giftItemReply( QString );
	void giftAddDownloadReply( QString );
	void giftAddUploadReply( QString );
	void giftAddSourceReply( QString );
	void giftDelSourceReply( QString );
	void giftChgUploadReply( QString );
	void giftChgDownloadReply( QString );
	void giftDelDownloadReply( QString );
	void giftDelUploadReply( QString );
	void giftShareReply( QString );

	void sigGiFTPID(pid_t);
};

#endif
